/*
Copyright (c) 2016, The `code.ivysaur.me/makedeb` author(s)

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above copyright
notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
*/

package makedeb

import (
	"io"
	"os"
	"time"
)

type DataContent []DataFile

func (this *DataContent) AddFile(LocalFile string, OutputPath string, FileMode int64, ModifyTime time.Time) {
	*this = append(*this, DataFile{
		LocalFile:  LocalFile,
		OutputPath: OutputPath,
		FileMode:   FileMode,
		ModifyTime: ModifyTime,
	})
}

func (this *DataContent) MakeTgz(output io.Writer) error {

	tgz, err := NewTarGz(output)
	if err != nil {
		return err
	}
	defer tgz.Close()

	for _, df := range *this {

		fstat, err := os.Stat(df.LocalFile)
		if err != nil {
			return err
		}

		fh, err := os.Open(df.LocalFile)
		if err != nil {
			return err
		}

		err = tgz.AddFile(fh, fstat.Size(), df.OutputPath, df.FileMode, df.ModifyTime)
		if err != nil {
			return err
		}

	}

	// All successful
	return nil
}
