/*
Copyright (c) 2016, The `code.ivysaur.me/makedeb` author(s)

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted, provided that the above copyright
notice and this permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
*/

package makedeb

import (
	"bytes"
	"io"
	"time"

	"github.com/qur/ar"
)

type DebPackage struct {
	ControlContent
	Files DataContent
}

func (this *DebPackage) Generate(output io.Writer) error {

	// ModTime

	currentTime := time.Now()

	// Ar file

	arw := ar.NewWriter(output)

	err := arw.WriteGlobalHeader()
	if err != nil {
		return err
	}

	// File: debian-binary

	err = arw.WriteHeader(&ar.Header{
		Name:    "debian-binary",
		Size:    4,
		Mode:    0644,
		ModTime: currentTime,
	})
	if err != nil {
		return err
	}

	_, err = arw.Write([]byte("2.0\n"))
	if err != nil {
		return err
	}

	// File: control.tar.gz

	controlTarGz := bytes.Buffer{}
	err = this.ControlContent.MakeTgz(&controlTarGz, currentTime)
	if err != nil {
		return err
	}

	err = arw.WriteHeader(&ar.Header{
		Name:    "control.tar.gz",
		Size:    int64(controlTarGz.Len()),
		Mode:    0644,
		ModTime: currentTime,
	})
	if err != nil {
		return err
	}

	_, err = io.Copy(arw, &controlTarGz)
	if err != nil {
		return err
	}

	// File: data.tar.gz

	dataTarGz := bytes.Buffer{}
	err = this.Files.MakeTgz(&dataTarGz)
	if err != nil {
		return err
	}

	err = arw.WriteHeader(&ar.Header{
		Name:    "data.tar.gz",
		Size:    int64(dataTarGz.Len()),
		Mode:    0644,
		ModTime: currentTime,
	})
	if err != nil {
		return err
	}

	_, err = io.Copy(arw, &dataTarGz)
	if err != nil {
		return err
	}

	// Done

	return nil

}
